from tcsoa.gen.Internal_EntCba._2019_12.services import AlignmentsService as imp0
from tcsoa.utils import TcService


class AlignmentsService(TcService):
    alignOccurrences = imp0.alignOccurrences
    unalignOccurrences = imp0.unalignOccurrences
