from tcsoa.gen.Internal_Security._2017_12.services import AwLicensingService as imp0
from tcsoa.utils import TcService


class AwLicensingService(TcService):
    getLicensesWithTypes = imp0.getLicensesWithTypes
