from tcsoa.gen.Internal_Gdis._2006_03.services import ReportingService as imp0
from tcsoa.utils import TcService


class ReportingService(TcService):
    getPrecalRoutineInfo = imp0.getPrecalRoutineInfo
