from tcsoa.gen.Internal_TCXMLImportExport._2020_12.services import BriefcaseService as imp0
from tcsoa.utils import TcService


class BriefcaseService(TcService):
    exportObjectsToBriefcase = imp0.exportObjectsToBriefcase
    exportObjectsToBriefcaseAsync = imp0.exportObjectsToBriefcaseAsync
    importObjectsFromBriefcase = imp0.importObjectsFromBriefcase
    importObjectsFromBriefcaseAsync = imp0.importObjectsFromBriefcaseAsync
