from tcsoa.gen.Internal_Structure._2020_12.services import WhereUsedService as imp0
from tcsoa.utils import TcService


class WhereUsedService(TcService):
    getWhereUsedInfo = imp0.getWhereUsedInfo
